// vim: ts=4 sw=4 expandtab cindent
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const flockIXmlRpcValue = Components.interfaces.flockIXmlRpcValue;
const flockIXmlRpcInt = Components.interfaces.flockIXmlRpcInt;
const flockIXmlRpcBoolean = Components.interfaces.flockIXmlRpcBoolean;
const flockIXmlRpcString = Components.interfaces.flockIXmlRpcString;
const flockIXmlRpcDouble = Components.interfaces.flockIXmlRpcDouble;
const flockIXmlRpcDateTime = Components.interfaces.flockIXmlRpcDateTime;
const flockIXmlRpcArray = Components.interfaces.flockIXmlRpcArray;
const flockIXmlRpcStruct = Components.interfaces.flockIXmlRpcStruct;
const flockIXmlRpcBase64 = Components.interfaces.flockIXmlRpcBase64;
const flockIXmlRpcServer = Components.interfaces.flockIXmlRpcServer;
const nsISupports = Components.interfaces.nsISupports;

/* Keep this object in sync between:
    browser/components/flock/xmlrpc/src/flockXmlRpcServer.js
   and:
    browser/components/flock/xmlrpc/public/xmlrpchelper.js
*/
function XmlRpcValue (aType) { this.XmlRpcType = aType; }
XmlRpcValue.prototype.QueryInterface = function (aIID) {
    const TYPES = [null, flockIXmlRpcInt, flockIXmlRpcBoolean, 
          flockIXmlRpcString, flockIXmlRpcDouble, flockIXmlRpcDateTime, 
          flockIXmlRpcArray, flockIXmlRpcStruct, flockIXmlRpcBase64];
    if (aIID.equals (flockIXmlRpcValue) || aIID.equals (nsISupports)) { 
        return this; 
    }
    if (this.XmlRpcType > 0 && this.XmlRpcType < TYPES.length && 
            aIID.equals (TYPES[this.XmlRpcType])) {
        return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
}
XmlRpcValue.Int = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_INT);
    value.Value = aValue;
    return value;
}
XmlRpcValue.Boolean = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_BOOLEAN);
    value.Value = aValue;
    return value;
}
XmlRpcValue.String = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_STRING);
    value.Value = aValue;
    return value;
}
XmlRpcValue.Double = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_DOUBLE);
    value.Value = aValue;
    return value;
}
XmlRpcValue.DateTime = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_DATETIME);
    value.Value = aValue.getTime() / 1000;
    return value;
}
XmlRpcValue.Array = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_ARRAY);
    value._value = aValue;
    value.elements = function (aCount) { 
        aCount.value = this._value.length;
        return this._value;
    };
    return value;
}
XmlRpcValue.Struct = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_STRUCT);
    value._value = aValue;
    value.keys = function (aCount) {
        var keys = [];
        for (var x in this._value) {
            keys.push (x);
        }
        aCount.value = keys.length;
        return keys;
    };
    value.get = function (aKey) {
        return this._value[aKey];
    };
    return value;
}
XmlRpcValue.Base64 = function (aValue) {
    var value = new XmlRpcValue (flockIXmlRpcValue.TYPE_BASE64);
    value.Value = aValue;
    return value;
}
XmlRpcValue.FromJavaScript = function (value) {
    switch (value.constructor) {
        case Number:
            if (Math.floor(value) != value) { 
                return this.Double (value); 
            } else { 
                return this.Int (value); 
            }
        case Boolean:
            return this.Boolean (value);
        case String:
            return this.String (value);
        case Date:
            return this.DateTime (value);
        case Array:
            var arr = new Array ();
            for (var i in value) {
                var v = this.FromJavaScript (value[i]);
                if (v != null && v != undefined) {
                    arr.push (v);
                }
            }
            return this.Array (arr);
        case Object:
            var obj = new Object ();
            for (var k in value) {
                var v = this.FromJavaScript (value[k]);
                if (v != null && v != undefined) {
                    obj[k] = v;
                }
            }
            return this.Struct (obj);
        case XmlRpcValue:
            // its already the right type
            return value;
        case Function:
            // ignore methods
            return null;
        default:
            throw 'error in XmlRpcValue.FromJavaScript converting ' +
                value.constructor;
    }
}
XmlRpcValue.ToJavaScript = function (value) {
    if (value == undefined) {
        return null;
    }
    if (value.QueryInterface) {
        value = value.QueryInterface (flockIXmlRpcValue);
        if (value == null) {
            return null;
        }
    } else {
        return null;
    }
    try {
        value = value.QueryInterface (flockIXmlRpcValue);
    } catch (e) {
        return value;
    }

    switch (value.XmlRpcType) {
        case flockIXmlRpcValue.TYPE_INT:
            value = value.QueryInterface (flockIXmlRpcInt);
            return value.Value;
        case flockIXmlRpcValue.TYPE_BOOLEAN:
            value = value.QueryInterface (flockIXmlRpcBoolean);
            return value.Value;
        case flockIXmlRpcValue.TYPE_STRING:
            value = value.QueryInterface (flockIXmlRpcString);
            return value.Value;
        case flockIXmlRpcValue.TYPE_DOUBLE:
            value = value.QueryInterface (flockIXmlRpcDouble);
            return value.Value;
        case flockIXmlRpcValue.TYPE_DATETIME:
            value = value.QueryInterface (flockIXmlRpcDateTime);
            var date = new Date ();
            date.setTime (value.Value * 1000);
            return date;
        case flockIXmlRpcValue.TYPE_ARRAY:
            value = value.QueryInterface (flockIXmlRpcArray);
            var array = new Array();
            var elements = value.elements ({});
            for (var i in elements) {
                array.push (XmlRpcValue.ToJavaScript (elements[i]));
            }
            return array;
        case flockIXmlRpcValue.TYPE_STRUCT:
            value = value.QueryInterface (flockIXmlRpcStruct);
            var keys = value.keys ({});
            var struct = {};
            for (var i in keys) {
                struct[keys[i]] = XmlRpcValue.ToJavaScript (value.get(keys[i]));
            }
            return struct;
        case flockIXmlRpcValue.TYPE_BASE64:
            value = value.QueryInterface (flockIXmlRpcBase64);
            return value.Value;
        default:
            throw 'error in XmlRpcValue.ToJavaScript';
    }
}



function flockXmlRpcServer (aURL) {
    this._server = Components.classes['@flock.com/xmlrpc/server;1']
        .createInstance (Components.interfaces.flockIXmlRpcServer);
    this._server.init (aURL);
}
flockXmlRpcServer.prototype.call = function (aMethod, aArguments, aListener) {
    var args = XmlRpcValue.FromJavaScript (aArguments);
    var listener = {
        onResult: function (aResult) {
            aListener.onResult (XmlRpcValue.ToJavaScript (aResult));
        },
        onFault: function (aFaultCode, aFaultString) {
            aListener.onFault (aFaultCode, aFaultString);
        },
        onError: function (aMessage) {
            aListener.onError (aMessage);
        },
    };
    this._server.call (aMethod, args, listener);
}
flockXmlRpcServer.prototype.makeProxy = function (aMethods) {
    var proxy = {__server: this};
    for (var i in aMethods) {
        var method_name = aMethods[i];
        var property = method_name.replace(/\./g, '_');
        proxy[property] = function () {
            if (arguments.length < 1) {
                throw 'not enough arguments';
            }
            proxy.__server.call (method_name, 
                    arguments.slice (0, arguments.length-1),
                    arguments[arguments.length-1]);
        }
    }
    return proxy;
}
