// vim: ts=4 sw=4 expandtab cindent foldmethod=marker
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// FIXME:
//   * moving XML parsing to E4X
//   * intelligent, incremental syncing down
//   * handle delicious failures intelligently

// CONSTANTS {{{
const flockIFavoritesWebService = Components.interfaces.flockIFavoritesWebService;
const flockIFavoritesChange = Components.interfaces.flockIFavoritesChange;
const flockIFavoritesChangeFavorite = Components.interfaces.flockIFavoritesChangeFavorite;
const flockIFavoritesQueue = Components.interfaces.flockIFavoritesQueue;
const flockIFavoritesUtils = Components.interfaces.flockIFavoritesUtils;
const flockIFavoriteProperties = Components.interfaces.flockIFavoriteProperties;
const nsISupports = Components.interfaces.nsISupports;
const nsITimer = Components.interfaces.nsITimer;
const nsITimerCallback = Components.interfaces.nsITimerCallback;
const nsIXMLHttpRequest = Components.interfaces.nsIXMLHttpRequest;
const nsIPasswordManager = Components.interfaces.nsIPasswordManager;
const nsIPassword = Components.interfaces.nsIPassword;
const nsIDOMSerializer = Components.interfaces.nsIDOMSerializer;
const nsIPrefBranch = Components.interfaces.nsIPrefBranch;
const nsIPromptService = Components.interfaces.nsIPromptService;

const TIMER_CONTRACTID = '@mozilla.org/timer;1';
const XMLHTTPREQUEST_CONTRACTID = '@mozilla.org/xmlextras/xmlhttprequest;1';
const PASSWORDMANAGER_CONTRACTID = '@mozilla.org/passwordmanager;1';
const XMLSERIALIZER_CONTRACTID = '@mozilla.org/xmlextras/xmlserializer;1';
const FFU_CONTRACTID = '@flock.com/favorites/utils;1';
const PREFERENCES_CONTRACTID = '@mozilla.org/preferences-service;1';
const PROMPT_CONTRACTID = '@mozilla.org/embedcomp/prompt-service;1';

const DIRECTION_TO_WEB_SERVICE = flockIFavoritesQueue.DIRECTION_TO_WEB_SERVICE;
const DIRECTION_FROM_WEB_SERVICE = flockIFavoritesQueue.DIRECTION_FROM_WEB_SERVICE;

const FIELD_WSID                 = flockIFavoriteProperties.FIELD_WSID;
const FIELD_TAGS                 = flockIFavoriteProperties.FIELD_TAGS;
const FIELD_PERMISSIONS          = flockIFavoriteProperties.FIELD_PERMISSIONS;
const FIELD_NAME                 = flockIFavoriteProperties.FIELD_NAME;
const FIELD_SHORTCUTURL          = flockIFavoriteProperties.FIELD_SHORTCUTURL;
const FIELD_DESCRIPTION          = flockIFavoriteProperties.FIELD_DESCRIPTION;
const FIELD_NSYNC                = flockIFavoriteProperties.FIELD_NSYNC;
const FIELD_LASTVISITDATE        = flockIFavoriteProperties.FIELD_LASTVISITDATE;
const FIELD_LASTMODIFIEDDATE     = flockIFavoriteProperties.FIELD_LASTMODIFIEDDATE;
const FIELD_ADDEDDATE            = flockIFavoriteProperties.FIELD_ADDEDDATE;

const DELICIOUS_MIN_WAIT = 1; // wait a minimum of two seconds between calls
const TIMEOUT = 10; // cancel requests that take more than 10 seconds 

const SEND_WAIT = 1; // wait one second between sending updates
const RECV_WAIT = 60*5; // wait 

const DELICIOUS_HOST = 'del.icio.us';
const DELICIOUS_PORT = 80;
const DELICIOUS_PATH = '/api/';

const PREF_USERNAME = 'flock.favorites.delicious.username';
const PREF_PASSWORD = 'flock.favorites.delicious.password';

const PREFS = Components.classes[PREFERENCES_CONTRACTID].getService (nsIPrefBranch);
/// }}}

// Date utility functions {{{

// seconds since epoch
function secondsSinceEpoch () { return (new Date()).getTime()/1000; }

const re_iso8601 = new RegExp(
    "^(\\d{4})(-(\\d{2})(-(\\d{2})" +
    "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
    "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$");
function parseDate(datestr) // {{{
{
    var date = null;

    datestr = datestr.replace(/^\s*|\s*$/g, '');

    var dateval = Date.parse(datestr);

    if (dateval) {
        date = new Date(dateval);
    } else {
        /* this is a parser for the W3C subset of ISO 8601 dates
         * http://www.w3.org/TR/NOTE-datetime
         */
        var m = datestr.match(re_iso8601);

        if (m) {
            date = new Date(Date.UTC(m[1]))

            m[3]  && date.setUTCMonth   (m[3] - 1);
            m[5]  && date.setUTCDate    (m[5]);
            m[7]  && date.setUTCHours   (m[7]);
            m[8]  && date.setUTCMinutes (m[8]);
            m[10] && date.setUTCSeconds (m[10]);

            m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

            if (m[14]) {
                var tz_off = m[16] * 60 + Number(m[17]);
                tz_off *= m[15] == '-' ? -1 : 1;
                tz_off *= 60 * 1000;

                date.setTime(date.getTime() - tz_off);
            }

        }
    }

    return date;
} // }}}

function isoDate (d) { // {{{
    function zeropad (s, l) {
        s = s.toString(); // force it to a string
        while (s.length < l) {
            s = '0' + s;
        }
        return s;
    }

    // if d is a number, turn it into a date
    if (typeof d == 'number') {
        var dt = new Date()
        dt.setTime (d);
        d = dt;
    }

    // YYYY-MM-DDThh:mm:ssZ
    return zeropad (d.getUTCFullYear (), 4) + '-' +
           zeropad (d.getUTCMonth (), 2) + '-' +
           zeropad (d.getUTCDate (), 2) + 'T' +
           zeropad (d.getUTCHours (), 2) + ':' +
           zeropad (d.getUTCMinutes (), 2) + ':' +
           zeropad (d.getUTCSeconds (), 2) + 'Z'
} // }}}

// }}}
// Delicious response parsing {{{
function parseResult (xml) { //{{{
    var results = xml.getElementsByTagName ('result');
    if (results.length != 1) {
        return null;
    }
    var code = results.item(0).attributes.getNamedItem ('code');
    return code.nodeValue;
} //}}}
function parsePosts (xml) { // {{{
    // so we want to parse into an array of flockIFavoritesChange

    var changeqi = function (aIID) {
        if (aIID.equals(flockIFavoritesChange) || 
                aIID.equals(flockIFavoritesChangeFavorite) || 
                aIID.equals(nsISupports)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }

    var changes = new Array ();

    var posts = new XML (Components.classes[XMLSERIALIZER_CONTRACTID]
            .createInstance (nsIDOMSerializer)
            .serializeToString(xml.documentElement));
    if (posts.name () != 'posts') {
        // the root element must be <posts>
        return null;
    }

    var ps = posts.post; // the list of <post> children
    for (var i in ps) {
        var post = ps[i];
        var properties = {
            fields: FIELD_NAME|FIELD_DESCRIPTION|FIELD_TAGS|FIELD_ADDEDDATE,
            name: post.@description.toString (),
            description: post.@extended.toString (),
            addedDate: parseDate(post.@time.toString ()).getTime(), 
        }
        var tags = post.@tag.toString ();
        if (tags == 'system:unfiled') {
            // this is a bogus tag we need to ignore
            tags = '';
        }
        /* translating from delicious back to flock tags:
            - replace spaces with commas
            - replace underscores with spaces
        */
        tags = tags.replace(/ /g, ',').replace(/_/g, ' ');
        // lets remove collection: tags for the time being
        tags = tags.split(/,/).filter (
                function (x) { return !x.match (/^collection:/); }).join (',');

        properties.tags = tags;
        changes.push ({
            url: post.@href.toString(),
            isDelete: false,
            properties: properties,
            QueryInterface: changeqi,
        });
    }

    return changes;
} //}}}
// }}}



// the DeliciousCall object wraps up everything involved in a call /*{{{*/
function DeliciousCall (noun, verb, args) {/*{{{*/
    dump ('DelicousCall ('+noun+','+verb+','+args+')\n');
    var url = noun+'/'+verb;
    var argsarray = new Array ();
    if (!args) {
        args = new Array ();
    }
    for (key in args) {
        dump (' '+key+'->'+args[key]+'\n');
        argsarray.push (encodeURIComponent(key) + '=' + 
                encodeURIComponent(args[key]));
    }
    if (argsarray.length) {
        url = url + '?' + argsarray.join('&');
    }
    dump ('url: '+url+'\n');
    this.url = url;
    this.request = null; // will be set to the XMLHttpRequest when its sent
    this.tries = 3;
}/*}}}*/
DeliciousCall.prototype.onError = function (message) {/*{{{*/
    dump ('DeliciousCall.onError ('+message+')\n');
}/*}}}*/
DeliciousCall.prototype.onResult = function (message, xml) {/*{{{*/
    dump ('DeliciousCall.onResult ('+message+','+xml+')\n');
}/*}}}*/
DeliciousCall.prototype.onTimeout = function () {/*{{{*/
    dump ('DeliciousCall.onTimeout ()\n');
}/*}}}*/
DeliciousCall.prototype.requestComplete = function () {/*{{{*/
    dump ('DeliciousCall.requestComplete\n');
    try {
        dump ('text='+this.request.responseText+'\n');
        var text = this.request.responseText;
        // strip invalid XML control characters from del.icio.us response
        // the only allowed control chars are \x09 \x0A \x0D
        text = text.replace (/[\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0B\x0C\x0E\x0F\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F]/g, '');
        var xml = Components.classes['@mozilla.org/xmlextras/domparser;1']
                .createInstance(Components.interfaces.nsIDOMParser)
                .parseFromString (text, 'text/xml');
        dump (' xml='+xml+'\n');
        dump (' this.request.responseText='+text+'\n');
        if (this.request.status == 200) {
            this.onResult (this.request.responseText, xml);
        } else {
            this.onError (this.request.statusMessage);
        }
    } catch (e) {
        this.onError ('exception '+e);
    }
}/*}}}*/
/*}}}*/

// the Delicious object manages a "connection" to a DelicousAPI server. {{{
// it handles queuing and throttling of requests and authentication
function Delicious (svc, url) {/*{{{*/
    if (url) {
        this.url = url;
    } else {
        this.url = 'http://del.icio.us/api/';
    }

    this.svc = svc;

    this.queue = new Array ();
    this.inProgress = false;
    this.inHTTP = false;
    this.lastCall = null;
    this.lastCallStarted = 0;
    this.lastCallEnded = 0;
    this.enabled = true;

    // set up a 1 second repeating timer
    this.timer = Components.classes[TIMER_CONTRACTID].createInstance(nsITimer);
    this.timer.initWithCallback (this, DELICIOUS_MIN_WAIT*1000, nsITimer.TYPE_REPEATING_SLACK);
}/*}}}*/
Delicious.prototype.username = function () {/*{{{*/
    return this.svc.username ();
}/*}}}*/
Delicious.prototype.password = function () {/*{{{*/
    return this.svc.password ();
}/*}}}*/
Delicious.prototype.setEnabled = function (aEnabled) {/*{{{*/
    this.enabled = aEnabled;
    if(!aEnabled) {
        this.queue = new Array();
    }
}/*}}}*/
Delicious.prototype.notify = function (timer) {/*{{{*/
    // nsITimer callback

    // dump ('timer callback\n');
    // dump (' inProgress = '+this.inProgress+'\n');
    // dump (' queue.length = '+this.queue.length+'\n');
    // dump (' lastCallEnded = '+this.lastCallEnded+'\n');
    // if we can send another request lets do it
    if (this.canSendRequest ()) {
        dump ('sending next request');
        this.sendNextRequest ();
        return;
    }

/* FIXME: timeout support in br0ken
    // if the current request has timed out 
    if (this.inHTTP && this.lastCallStarted 
            && (this.lastCallStarted+TIMEOUT > secondsSinceEpoch())) {
        // lets cancel it
        dump ('timed out after '+(secondsSinceEpoch()-this.lastCallStarted)+' seconds (timeout: '+TIMEOUT+'s)\n');
        dump (' '+(this.lastCallStarted+TIMEOUT)+' > '+secondsSinceEpoch()+'\n');

        this.lastCall.request.abort ();

        dump ('calling onTimeout()\n');
        // pass an error back
        if (this.lastCall) {
            this.lastCall.onTimeout ()
        }

        // and get ready for the next call
        this.lastCall = null;
        this.lastCallEnded = secondsSinceEpoch ();
        this.inProgress = false;
        dump ('\n\n\n\n\n\n\n');
        return;
    }
*/
}/*}}}*/
Delicious.prototype.submit = function (call) {/*{{{*/
    dump ('Delicious.submit ('+call+')\n');
    call.tries--;
    if (call.tries < 0) {
        // tried too many times
        dump ('Delicious.submit - giving up\n');
        return;
    }
    // add this call to the queue
    this.queue.push (call);
}/*}}}*/
Delicious.prototype.canSendRequest = function () {/*{{{*/
    return !this.inProgress 
        && (this.queue.length > 0)
        && this.enabled
        && (this.lastCallEnded+DELICIOUS_MIN_WAIT<=secondsSinceEpoch());
}/*}}}*/
Delicious.prototype.sendNextRequest = function () {/*{{{*/
    dump ('Delicious.sendNextRequest\n');
    // now we have a call in progress
    this.inProgress = true;
    this.lastCallStarted = secondsSinceEpoch ();

    // get the call
    this.lastCall = this.queue.shift ();

    // build the request
    this.inHTTP = true;
    this.lastCall.request = {};
    var delicious = this;

    var headers = {}
    headers['Authorization'] = 'Basic '+ 
        httpClient.base64 (this.username()+':'+this.password());
    headers['User-Agent'] = 'Flock Delicious Client';
        

    httpClient.GET (DELICIOUS_HOST, DELICIOUS_PORT, 
            DELICIOUS_PATH+this.lastCall.url, headers,
            function (status, message, headers, body) {
                dump (' complete!\n');
                delicious.inHTTP = false;
                // FIXME: handle 503
                // call the callbacks

                delicious.lastCall.request.status = status;
                delicious.lastCall.request.statusMessage = message;
                delicious.lastCall.request.responseText = body;
                delicious.lastCall.requestComplete ();
                // and clean up
                delicious.lastCall = null;
                delicious.lastCallEnded = secondsSinceEpoch ();
                delicious.inProgress = false;
                dump ('\n\n\n\n\n\n\n');

                // XXX HACK auth failed - prompt
                if (status == 401) {
                    delicious.svc.promptForLogin ();
                }
            });

    // FIXME: change user agent?
    if (this.lastCall.listener) this.lastCall.listener.onStart();
}/*}}}*/
// }}}

// The deliciousWebService flockIFavoritesWebService implementation {{{
function deliciousWebService () {/*{{{*/
    dump ('constructed deliciousWebService\n');
    this.mQueue = null;

    this.mDelicious = new Delicious (this);

    this.mLastUpdate = 0;

    this.mSendTimer = Components.classes[TIMER_CONTRACTID]
        .createInstance(nsITimer);
    this.mSendTimer.initWithCallback (this, SEND_WAIT*1000, 
            nsITimer.TYPE_REPEATING_SLACK);

    // set attributes
    this.configurationDialog = null;
    this.name = 'del.icio.us';
    this.description = 'A leading social bookmarking service';
    this.id = 'delicious';
    this.topbarPage = 'chrome://browser/content/flock/favorites/deliciousTopbarPage.xul';
    this.supportedFields = FIELD_NAME|FIELD_DESCRIPTION|FIELD_TAGS|FIELD_ADDEDDATE;
    this.supportsCollections = false;
    this.supportsExplicitDeletes = false;
}/*}}}*/
deliciousWebService.prototype.QueryInterface = function (iid) {/*{{{*/
    if (iid.equals(flockIFavoritesWebService) || 
            iid.equals(nsITimerCallback) ||
            iid.equals(nsISupports)) {
        return this;
    }

    throw Components.results.NS_ERROR_NO_INTERFACE;
}/*}}}*/
deliciousWebService.prototype.promptForLogin = function () { // {{{
    var svc = Components.classes[PROMPT_CONTRACTID]
        .getService (nsIPromptService);
    var un = '';
    var pw = '';
    try {
        un = PREFS.getCharPref (PREF_USERNAME);
    } catch (e) { }
    try {
        pw = PREFS.getCharPref (PREF_PASSWORD);
    } catch (e) { }
    var u = { value: un }
    var p = { value: pw }
    var ok = svc.promptUsernameAndPassword (null, 'del.icio.us Login Details', 
            'Enter your del.icio.us user name and password', u, p, null, {value:0});
    if (!ok) {
        u.value = '';
        p.value = '';
    }
    PREFS.setCharPref (PREF_USERNAME, u.value);
    PREFS.setCharPref (PREF_PASSWORD, p.value);
} // }}}    
deliciousWebService.prototype.username = function () { // {{{
    var un = null;
    while (un == null) {
        // try to get the preference
        try {
            un = PREFS.getCharPref (PREF_USERNAME);
        } catch (e) { }

        if (un == null) {
            this.promptForLogin ();
        }
    }
    return un;
} // }}}
deliciousWebService.prototype.password = function () { // {{{
    var pw = null;
    while (pw == null) {
        // try to get the preference
        try {
            pw = PREFS.getCharPref (PREF_PASSWORD);
        } catch (e) { }

        if (pw == null) {
            this.promptForLogin ();
        }
    }
    return pw;
} // }}}
// nsITimerCallback implementation
deliciousWebService.prototype.notify = function (aTimer) {//{{{
    var queue = this.mQueue;
    if (aTimer == this.mSendTimer) {
        if (this.mDelicious.inCall || this.mDelicious.queue.length) {
            return;
        }
        if (!this.mQueue) {
            return;
        }


        var change = this.mQueue.nextChange (DIRECTION_TO_WEB_SERVICE);
        if (change == null) {
            // no change pending
            return;
        }

        try {
            change = change.QueryInterface (flockIFavoritesChangeFavorite);
            if (change == null) {
                // not the right kind of change
                return;
            }
        } catch (e) {
            // not the right kind of change
            return;
        }

        if (change.isDelete) {
            var call = new DeliciousCall ('posts', 'delete', {url: change.url});
            call.onError = function (message) { 
                // re-queue failed request
                queue.deleteFavorite (change.url, change.properties, 
                        DIRECTION_TO_WEB_SERVICE);
            }
            call.onResult = function (message, xml) { }
            call.onTimeout = function () { 
                // re-queue failed request
                queue.deleteFavorite (change.url, change.properties, 
                        DIRECTION_TO_WEB_SERVICE);
            }
            this.mDelicious.submit (call);
        } else {
            var ffu = Components.classes[FFU_CONTRACTID]
                .getService (flockIFavoritesUtils);

            dump ('adding: '+ffu.propertiesToString (change.properties)+'\n');
            var args = { replace: 'yes', url: change.url, tags: '' };
            if (change.properties.fields & FIELD_NAME) {
                args.description = change.properties.name;
            }
            if (change.properties.fields & FIELD_DESCRIPTION) {
                args.extended = change.properties.description;
            }
            if (change.properties.fields & FIELD_TAGS) {
                /* the translation is something like this: 
                     - remove spaces surrounding commas
                     - turn spaces into pluses
                     - turn commas into spaces
                 */
                args.tags = change.properties.tags.
                    // remove extra whitespace
                    replace(/^ */,'').replace(/ *$/,'').replace(/ *, */g,',').
                    // replace spaces with _ and commas with spaces
                    replace(/ /g,'_').replace(/,/g, ' ');
            }
/* Actually, don't send the date - its screwing things up and is unnececary,
   the server will set it.
            if (change.properties.fields & FIELD_ADDEDDATE) {
                args.time = isoDate (change.properties.addedDate);
            }
*/
            var call = new DeliciousCall ('posts', 'add', args);
            call.onError = function (message) { 
                // re-queue failed request
                queue.changeFavorite (change.url, change.properties, 
                        DIRECTION_TO_WEB_SERVICE);
            }
            call.onResult = function (message, xml) { 
                //result = parseDeliciousResponse (xml);
            }
            call.onTimeout = function () { 
                // re-queue failed request
                queue.changeFavorite (change.url, change.properties, 
                        DIRECTION_TO_WEB_SERVICE);
            }

            this.mDelicious.submit (call);
        }

    } else {
        dump ('callback for unknown timer: '+aTimer+'\n');
    }
}//}}}


// flockIFavoritesWebService implementation
deliciousWebService.prototype.setFavoritesQueue = function (aQueue) {/*{{{*/
    dump ('deliciousWebService.setFavoritesQueue('+aQueue+')\n');
    this.mQueue = aQueue;
}/*}}}*/
deliciousWebService.prototype.getAllFavorites = function (aListener) {/*{{{*/
    dump ('deliciousWebService.getAllFavorites('+aListener+')\n');
    var service = this;
    var call = new DeliciousCall ('posts', 'all');
    call.onError = function (message) {
        aListener.onFailed (message);
    }
    call.onResult = function (message, xml) { 
        var changes = parsePosts (xml);
        if (changes == null) {
            aListener.onFailed ('failed to parse posts/all response');
            return;
        }
        // turn the changes array into an nsISimpleEnumerator
        changes.hasMoreElements = function () { return this.length != 0; }
        changes.getNext = function () { return this.shift (); }
        aListener.onSucceeded (changes);
    }
    call.onTimeout = function () {
        aListener.onFailed ('Timeout');
    }
    this.mDelicious.submit (call);
}/*}}}*/
deliciousWebService.prototype.getSyncRequired = function (aListener) {/*{{{*/
    dump ('deliciousWebService.getSyncRequired()\n');
    var service = this;
    var call = new DeliciousCall ('posts', 'update');
    call.onError = function (message) {
        aListener.onFailed (message);
    }
    call.onResult = function (message, xml) { 
        var updates_xml = xml.getElementsByTagName ('update');
        if (updates_xml == null || updates_xml.length !=1) {
            aListener.onFailed ('failed to parse update response');
            return;
        }
        var update_xml = updates_xml.item (0);
        var attrs = update_xml.attributes;
        for (var i=0; i<attrs.length; i++) {
            if (attrs[i].nodeName == 'time') {
                var update = parseDate (attrs[i].nodeValue);
                if (update == null) {
                    aListener.onFailed ('failed to parse update date');
                    return;
                }
                if (update > service.mLastUpdate) {
                    // there's been a change
                    service.mLastUpdate = update;
                    aListener.onSucceeded (true);
                    return
                }
                // no change
                aListener.onSucceeded (false);
                return
            }
        }
        aListener.onFailed ('failed to parse update response');
    }
    call.onTimeout = function () {
        aListener.onFailed ('Timeout');
    }
    this.mDelicious.submit (call);
}/*}}}*/
deliciousWebService.prototype.__defineGetter__('webView', function() { // {{{
    dump ('deliciousWebService.webView getter\n');
    return 'http://del.icio.us/'+this.username()+'/';

});/*}}}*/
deliciousWebService.prototype.login = function (aUserName, aPassword, aListener) {/*{{{*/
    dump ('deliciousWebService.login("'+aUserName+'", "'+aPassword+'", '+aListener+')\n');
    PREFS.setCharPref (PREF_USERNAME, aUserName);
    PREFS.setCharPref (PREF_PASSWORD, aPassword);

    var service = this;
    var call = new DeliciousCall ('posts', 'update');
    call.onError = function (message) {
        aListener.onFailed (message);
    }
    call.onResult = function (message, xml) { 
        aListener.onSucceeded ();
    }
    call.onTimeout = function () {
        aListener.onFailed ('Timeout');
    }
    this.mDelicious.submit (call);
}/*}}}*/
// }}}

// JavaScript XPCOM Boilerplate {{{

// XPCOM Boilerplate {{{
function loadLibrary(aFilename) {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
        .getService(Components.interfaces.nsIProperties)
        .get("ComsD", Components.interfaces.nsIFile);
    file.append(aFilename);

    var ios = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
        .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var spec = fileHandler.getURLSpecFromFile(file);

    var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
        .getService(Components.interfaces.mozIJSSubScriptLoader);

    scriptLoader.loadSubScript(spec);
}
loadLibrary("flockXPCOMTemplate.js.lib");
// }}}

var module = new xpcomModule ('{26fec785-bb4d-4695-bd8e-6f27f6dc2d49}',
        '@flock.com/favorites/service/delicious;1', 
        'Delicious Favorites Web Service', 
        deliciousWebService);
module.setFavoritesWebService ('delicious-favorites-service');
function NSGetModule(aCompMgr, aFileSpec) {
    return module;
};
// }}}

function loadLibraryFromSpec(spec)
{
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript(spec);
}
loadLibraryFromSpec('chrome://browser/content/flock/common/httpClient.js');

// enable logging
httpClient.log = function (msg) { dump ('DELICIOUS-HTTP: '+msg); }
